package cs214.webapp.server

import web.WebServer
import cs214.webapp.server.utils.AppsLoader
import cs214.webapp.Config.HTTP_PORT

/** Main entry point of the server of an application. Starts the server on
  * construction, call `start()` to initialize the apps.
  */
abstract class ApplicationJVM extends cask.Main:

  override def host: String = "0.0.0.0"
  override def port: Int = HTTP_PORT
  val allRoutes = Seq(web.WebServerRoutes())
  println(s"[WebServer] started on ${host}:${port}.")

  def start(): Unit =
    // Find all apps and register them.
    val classes = AppsLoader.loadApps()

    classes.foreach: candidateClass =>
      val appLogicInstance = candidateClass
        .getDeclaredConstructor()
        .newInstance()
        .asInstanceOf[StateMachine[?, ?, ?]]

      val appPackageName = appLogicInstance.getClass.getPackageName.replaceFirst("apps\\.", "")
      require(
        appPackageName.contains(appLogicInstance.appInfo.id),
        s"Found invalid appId: Package name is \"$appPackageName\"" +
          s" but id specified in app logic is \"${appLogicInstance.appInfo.id}\"." +
          s" These values should match!"
      )
      WebServer.register(appLogicInstance)
