package cs214.webapp

import ujson.Value

import scala.util.Try

case class AppInfo(
    /** A lowercase letters ([a-z]) only app identifier (for example
      * "tictactoe")
      */
    id: AppId,
    /** The display name of the application (for example "Tic Tac Toe"), less
      * than 32 characters
      */
    name: String,
    /** A description of the application, less than 160 characters */
    description: String,
    /** The current year for the course */
    year: Int
):
  private val MAX_APP_DESC_LEN = 160
  private val MAX_APP_NAME_LEN = 32
  private val ID_PATTERN = "^[a-z_0-9]+$"

  require(id.matches(ID_PATTERN), f"App id \"$id\" is invalid. It should match this pattern: $ID_PATTERN")
  require(name.length <= MAX_APP_NAME_LEN, s"App name \"$name\" is too long (> $MAX_APP_NAME_LEN chars)")
  require(
    description.length <= MAX_APP_DESC_LEN,
    s"App description \"${description}\" is too long (> $MAX_APP_DESC_LEN chars)"
  )

/** Provides all the necessary encoding/decoding methods for an application. */
trait AppWire[Event, View]:
  val eventFormat: WireFormat[Event]
  val viewFormat: WireFormat[View]

case class Tick(systemMillis: Long)

object TickEventFormat extends WireFormat[Tick]:
  override def decode(json: Value): Try[Tick] =
    Try:
      Tick(json.obj.get("millis").get.num.toLong)

  override def encode(t: Tick): Value = ujson.Obj("type" -> "Tick", "millis" -> ujson.Num(t.systemMillis.toDouble))
